"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const extend = require("deep-extend");
const fs = require("fs");
const path = require("path");
const conf_1 = require("../services/conf");
const logger_1 = require("../services/logger");
const render_1 = require("../services/render");
const sync_1 = require("../services/sync");
const watcher_1 = require("../services/watcher");
const util_1 = require("../util");
const { resolve } = path;
const { templateDir, rdeDir, runtimeDir, dockerWorkDirRoot, RdTypes, } = conf_1.default;
class DockerRun {
    constructor({ watch }) {
        this.watch = watch;
    }
    async start() {
        // merge rda.config rdc config
        this.config = await this.mergeRdcConf();
        if (conf_1.default.isIntegrate) {
            // template render to .integrated
            await this.renderDir(templateDir, rdeDir);
            // compose rde and watch app
            await this.composeRde();
        }
        else {
            // template render to runtime
            await this.renderDir(templateDir, runtimeDir);
        }
        if (conf_1.default.rdType === RdTypes.Application) {
            await sync_1.default.mergePkgJson(resolve(dockerWorkDirRoot, 'app', 'package.json'), resolve(dockerWorkDirRoot, templateDir, 'package.json'), resolve(dockerWorkDirRoot, runtimeDir));
        }
        if (this.watch && conf_1.default.rdType === RdTypes.Container) {
            await this.watchTemplate(conf_1.default.isIntegrate ? rdeDir : runtimeDir);
        }
    }
    async mergeRdcConf() {
        const { cwd, appConfName, rdcConfName, rdeDir, } = conf_1.default;
        let config = require(resolve(cwd, rdcConfName));
        const rdaConfPath = resolve(cwd, appConfName);
        if (fs.existsSync(rdaConfPath)) {
            config = extend(config, require(rdaConfPath));
        }
        await render_1.default.renderTo('module', {
            obj: config
        }, resolve(rdeDir, rdcConfName), {
            overwrite: true
        });
        conf_1.default.rdMode = config.mode || conf_1.default.RdModes.Integrate;
        return config;
    }
    async renderDir(from, to) {
        // @ts-ignore
        const { render: rdtRender = {}, container, suites } = this.config;
        const { includes = [] } = rdtRender;
        if (suites && suites.length) {
            container.render.suites = suites;
        }
        const dataView = container ? container.render : rdtRender.mock || {};
        await render_1.default.renderDir(from, Object.assign({}, dataView), includes, to, {
            overwrite: true,
            dot: true
        });
    }
    async composeRde() {
        logger_1.debug(`merged mappings: ${JSON.stringify(this.config.mappings)}`);
        for (let { from, to, options } of this.config.mappings) {
            const appDir = resolve(conf_1.default.cwd, from);
            const destDir = resolve(rdeDir, to);
            await util_1.default.copy(appDir, destDir, { options });
        }
        if (this.watch) {
            let { mappings } = this.config;
            new watcher_1.default(mappings).start();
        }
    }
    async watchTemplate(dest) {
        const mappings = [{
                from: templateDir,
                to: dest
            }];
        new watcher_1.default(mappings, this.renderDir.bind(this)).start();
    }
}
exports.default = DockerRun;
